from dd_import.dd_api import Api
from dd_import.environment import Environment


def dd_import_languages():
    try:
        environment = Environment()
        environment.check_environment_languages()
        api = Api()
        product_type_id = api.get_product_type()
        product_id = api.get_product(product_type_id)
        api.import_languages(product_id)
    except Exception as e:
        print(str(e))
        exit(1)


if __name__ == '__main__':
    dd_import_languages()



from dd_import.dd_api import Api
from dd_import.environment import Environment


def dd_reimport_findings():
    try:
        environment = Environment()
        environment.check_environment_reimport_findings()
        api = Api()
        product_type_id = api.get_product_type()
        product_id = api.get_product(product_type_id)
        engagement_id = api.get_engagement(product_id)
        test_id = api.get_test(engagement_id)
        api.reimport_scan(test_id)
        api.update_engagement(engagement_id)
    except Exception as e:
        print(str(e))
        exit(1)


if __name__ == '__main__':
    dd_reimport_findings()



import datetime
import os
from distutils.util import strtobool


class Environment:

    def __init__(self):
        self.url = os.getenv('DD_URL')
        self.api_key = os.getenv('DD_API_KEY')
        self.product_name = os.getenv('DD_PRODUCT_NAME')
        self.product_type_name = os.getenv('DD_PRODUCT_TYPE_NAME')
        self.engagement_name = os.getenv('DD_ENGAGEMENT_NAME')
        self.engagement_target_start = os.getenv('DD_ENGAGEMENT_TARGET_START', datetime.date.today().isoformat())
        self.engagement_target_end = os.getenv('DD_ENGAGEMENT_TARGET_END', '2999-12-31')
        self.test_name = os.getenv('DD_TEST_NAME')
        self.test_type_name = os.getenv('DD_TEST_TYPE_NAME')
        self.file_name = os.getenv('DD_FILE_NAME')
        self.active = os.getenv('DD_ACTIVE', 'True').lower() in ['true']
        self.verified = os.getenv('DD_VERIFIED', 'True').lower() in ['true']
        self.minimum_severity = os.getenv('DD_MINIMUM_SEVERITY', None)
        self.group_by = os.getenv('DD_GROUP_BY', None)
        self.push_to_jira = os.getenv('DD_PUSH_TO_JIRA', 'False').lower() in ['true']
        self.close_old_findings = os.getenv('DD_CLOSE_OLD_FINDINGS', 'True').lower() in ['true']
        self.version = os.getenv('DD_VERSION', None)
        self.endpoint_id = os.getenv('DD_ENDPOINT_ID', None)
        self.service = os.getenv('DD_SERVICE', None)
        self.build_id = os.getenv('DD_BUILD_ID', None)
        self.commit_hash = os.getenv('DD_COMMIT_HASH', None)
        self.branch_tag = os.getenv('DD_BRANCH_TAG', None)
        self.api_scan_configuration_id = os.getenv('DD_API_SCAN_CONFIGURATION_ID', None)
        self.ssl_verification = bool(strtobool(os.getenv('DD_SSL_VERIFY', 'true')))

    def check_environment_reimport_findings(self):
        error_string = self.check_environment_common()
        if self.engagement_name is None:
            if error_string != '':
                error_string = error_string + ' / '
            error_string = error_string + 'DD_ENGAGEMENT_NAME is missing'
        if self.test_name is None:
            if error_string != '':
                error_string = error_string + ' / '
            error_string = error_string + 'DD_TEST_NAME is missing'
        if self.test_type_name is None:
            if error_string != '':
                error_string = error_string + ' / '
            error_string = error_string + 'DD_TEST_TYPE_NAME is missing'

        if len(error_string) > 0:
            raise Exception(error_string)

        print('DD_URL:                       ', self.url)
        print('DD_PRODUCT_TYPE_NAME:         ', self.product_type_name)
        print('DD_PRODUCT_NAME:              ', self.product_name)
        print('DD_ENGAGEMENT_NAME:           ', self.engagement_name)
        print('DD_ENGAGEMENT_TARGET_START:   ', self.engagement_target_start)
        print('DD_ENGAGEMENT_TARGET_END:     ', self.engagement_target_end)
        print('DD_TEST_NAME:                 ', self.test_name)
        print('DD_TEST_TYPE_NAME:            ', self.test_type_name)
        print('DD_FILE_NAME:                 ', self.file_name)
        print('DD_ACTIVE:                    ', self.active)
        print('DD_VERIFIED:                  ', self.verified)
        print('DD_MINIMUM_SEVERITY:          ', self.minimum_severity)
        print('DD_GROUP_BY:                  ', self.group_by)
        print('DD_PUSH_TO_JIRA:              ', self.push_to_jira)
        print('DD_CLOSE_OLD_FINDINGS:        ', self.close_old_findings)
        print('DD_VERSION:                   ', self.version)
        print('DD_ENDPOINT_ID:               ', self.endpoint_id)
        print('DD_SERVICE:                   ', self.service)
        print('DD_BUILD_ID:                  ', self.build_id)
        print('DD_COMMIT_HASH:               ', self.commit_hash)
        print('DD_BRANCH_TAG:                ', self.branch_tag)
        print('DD_API_SCAN_CONFIGURATION_ID: ', self.api_scan_configuration_id)
        print('DD_SSL_VERIFY:                ', self.ssl_verification)
        print('')

    def check_environment_languages(self):
        error_string = self.check_environment_common()

        if self.file_name is None:
            if error_string != '':
                error_string = error_string + ' / '
            error_string = error_string + 'DD_FILE_NAME is missing'

        if len(error_string) > 0:
            raise Exception(error_string)

        print('DD_URL:                ', self.url)
        print('DD_PRODUCT_TYPE_NAME:  ', self.product_type_name)
        print('DD_PRODUCT_NAME:       ', self.product_name)
        print('DD_FILE_NAME:          ', self.file_name)
        print('DD_SSL_VERIFY:         ', self.ssl_verification)
        print('')

    def check_environment_common(self):
        error_string = ''
        if self.url is None:
            error_string = 'DD_URL is missing'
        if self.api_key is None:
            if error_string != '':
                error_string = error_string + ' / '
            error_string = error_string + 'DD_API_KEY is missing'
        if self.product_type_name is None:
            if error_string != '':
                error_string = error_string + ' / '
            error_string = error_string + 'DD_PRODUCT_TYPE_NAME is missing'
        if self.product_name is None:
            if error_string != '':
                error_string = error_string + ' / '
            error_string = error_string + 'DD_PRODUCT_NAME is missing'

        return error_string



import datetime
import json

import requests
from urllib3 import disable_warnings
from urllib3.exceptions import InsecureRequestWarning

from dd_import.environment import Environment

# Disable SSL Warnings
disable_warnings(InsecureRequestWarning)


# Fix for reimport without file, see https://github.com/psf/requests/issues/1081#issuecomment-428504128
class ForceMultipartDict(dict):
    def __bool__(self):
        return True


FORCE_MULTIPART = ForceMultipartDict()  # An empty dict that boolean-evaluates as `True`.


class Api:

    def __init__(self):
        self.environment = Environment()
        self.headers = {'Content-type': 'application/json',
                        'Authorization': 'Token ' + self.environment.api_key}
        self.headers_without_json = {'Authorization':
                                     'Token ' + self.environment.api_key}
        self.product_type_url = self.environment.url + '/api/v2/product_types/'
        self.product_url = self.environment.url + '/api/v2/products/'
        self.engagement_url = self.environment.url + '/api/v2/engagements/'
        self.test_url = self.environment.url + '/api/v2/tests/'
        self.test_type_url = self.environment.url + '/api/v2/test_types/'
        self.reimport_scan_url = self.environment.url + '/api/v2/reimport-scan/'
        self.import_languages_url = self.environment.url + '/api/v2/import-languages/'
        self.ssl_verification = self.environment.ssl_verification

    def get_product_type(self):
        payload = {'name': self.environment.product_type_name}
        r = requests.get(self.product_type_url,
                         headers=self.headers,
                         params=payload,
                         verify=self.ssl_verification)
        r.raise_for_status()
        product_type_data = json.loads(r.text)
        for product_type in product_type_data.get('results', []):
            if product_type.get('name', '') == self.environment.product_type_name:
                product_type_id = product_type['id']
                print('Product type found, id: ', product_type_id)
                return product_type_id
        raise Exception(f'Product type {self.environment.product_type_name} not found')

    def get_product(self, product_type):
        payload = {'name': self.environment.product_name,
                   'prod_type': product_type}
        r = requests.get(self.product_url,
                         headers=self.headers,
                         params=payload,
                         verify=self.ssl_verification)
        r.raise_for_status()
        product_data = json.loads(r.text)
        for product in product_data.get('results', []):
            if product.get('name', '') == self.environment.product_name:
                product_id = product['id']
                print('Product found,      id: ', product_id)
                return product_id
        return self.new_product(product_type)

    def new_product(self, product_type):
        payload = {'name': self.environment.product_name,
                   'description': self.environment.product_name,
                   'prod_type': product_type}
        r = requests.post(self.product_url,
                          headers=self.headers,
                          data=json.dumps(payload),
                          verify=self.ssl_verification)
        r.raise_for_status()
        product_data = json.loads(r.text)
        print('New product,        id: ', product_data['id'])
        return product_data['id']

    def get_engagement(self, product):
        payload = {'name': self.environment.engagement_name,
                   'product': product}
        r = requests.get(self.engagement_url,
                         headers=self.headers,
                         params=payload,
                         verify=self.ssl_verification)
        r.raise_for_status()
        engagement_data = json.loads(r.text)
        for engagement in engagement_data.get('results', []):
            if engagement.get('name', '') == self.environment.engagement_name:
                engagement_id = engagement['id']
                print('Engagement found,   id: ', engagement_id)
                return engagement_id
        return self.new_engagement(product)

    def new_engagement(self, product):
        payload = {'name': self.environment.engagement_name,
                   'product': product,
                   'target_start': self.environment.engagement_target_start,
                   'target_end': self.environment.engagement_target_end,
                   'engagement_type': 'CI/CD',
                   'status': 'In Progress'}
        r = requests.post(self.engagement_url,
                          headers=self.headers,
                          data=json.dumps(payload),
                          verify=self.ssl_verification)
        r.raise_for_status()
        engagement_data = json.loads(r.text)
        print('New engagement,     id: ', engagement_data['id'])
        return engagement_data['id']

    def update_engagement(self, engagement):
        if self.environment.build_id is not None or \
           self.environment.commit_hash is not None or \
           self.environment.branch_tag is not None:
            payload = {'build_id': self.environment.build_id,
                       'commit_hash': self.environment.commit_hash,
                       'branch_tag': self.environment.branch_tag}
            r = requests.patch(self.engagement_url + str(engagement) + '/',
                               headers=self.headers,
                               data=json.dumps(payload),
                               verify=self.ssl_verification)
            r.raise_for_status()

    def get_test(self, engagement):
        payload = {'title': self.environment.test_name,
                   'engagement': engagement}
        r = requests.get(self.test_url,
                         headers=self.headers,
                         params=payload,
                         verify=self.ssl_verification)
        r.raise_for_status()
        test_data = json.loads(r.text)
        for test in test_data.get('results', []):
            if test.get('title', '') == self.environment.test_name:
                test_id = test['id']
                print('Test found,         id: ', test_id)
                return test_id
        return self.new_test(engagement)

    def new_test(self, engagement):
        today = datetime.date.today()
        today = datetime.datetime(today.year, today.month, today.day)
        payload = {'title': self.environment.test_name,
                   'engagement': engagement,
                   'target_start': today.isoformat(),
                   'target_end': datetime.datetime
                   .fromisoformat('2999-12-31').isoformat(),
                   'test_type': self.get_test_type()}
        r = requests.post(self.test_url,
                          headers=self.headers,
                          data=json.dumps(payload),
                          verify=self.ssl_verification)
        r.raise_for_status()
        test_data = json.loads(r.text)
        print('New test,           id: ', test_data['id'])
        return test_data['id']

    def get_test_type(self):
        payload = {'name': self.environment.test_type_name}
        r = requests.get(self.test_type_url,
                         headers=self.headers,
                         params=payload,
                         verify=self.ssl_verification)
        r.raise_for_status()
        test_type_data = json.loads(r.text)
        for test_type in test_type_data.get('results', []):
            if test_type.get('name', '') == self.environment.test_type_name:
                return test_type['id']
        raise Exception(f'Test type {self.environment.test_type_name} not found')

    def reimport_scan(self, test):
        payload = {'scan_date': datetime.date.today().isoformat(),
                   'scan_type': self.environment.test_type_name,
                   'test': test,
                   'active': self.environment.active,
                   'verified': self.environment.verified,
                   'push_to_jira': self.environment.push_to_jira,
                   'close_old_findings': self.environment.close_old_findings,
                   }
        if self.environment.minimum_severity is not None:
            payload['minimum_severity'] = self.environment.minimum_severity
        if self.environment.group_by is not None:
            payload['group_by'] = self.environment.group_by
        if self.environment.version is not None:
            payload['version'] = self.environment.version
        if self.environment.endpoint_id is not None:
            payload['endpoint_to_add'] = int(self.environment.endpoint_id)
        if self.environment.service is not None:
            payload['service'] = self.environment.service
        if self.environment.api_scan_configuration_id is not None:
            payload['api_scan_configuration'] = self.environment.api_scan_configuration_id

        if self.environment.file_name is not None:
            files = {'file': (self.environment.file_name,
                              open(self.environment.file_name, 'rb'),
                              'application/json', {'Expires': '0'})}
            response = requests.post(self.reimport_scan_url,
                                     headers=self.headers_without_json,
                                     data=payload,
                                     files=files,
                                     verify=self.ssl_verification)
        else:
            response = requests.post(self.reimport_scan_url,
                                     headers=self.headers_without_json,
                                     data=payload,
                                     files=FORCE_MULTIPART,
                                     verify=self.ssl_verification)

        response.raise_for_status()

        print()
        print('Scan results imported')

    def import_languages(self, product):
        payload = {'product': product}
        files = {'file': (self.environment.file_name,
                 open(self.environment.file_name, 'rb'),
                 'application/json', {'Expires': '0'})}

        response = requests.post(self.import_languages_url,
                                 headers=self.headers_without_json,
                                 data=payload,
                                 files=files,
                                 verify=self.ssl_verification)
        response.raise_for_status()

        print()
        print('Languages imported')
